import 'dart:typed_data';

import 'package:bip39/bip39.dart' as bip39;
import 'package:cryptography/cryptography.dart';
import 'package:ed25519_hd_key/ed25519_hd_key.dart';
import 'package:solana_dart/src/base58/base58.dart' as base58;

/// Signs transactions to be sent to a Solana Cluster
class SolanaWallet {
  SolanaWallet._fromKeyPair(KeyData data)
      : _keyData = data,
        address = base58.encode(
            ED25519_HD_KEY.getPublicKey(data.key as Uint8List).sublist(0, 32)) {
    print(address);
  }

  /// Creates and initializes a new SolanaWallet for the given bip39
  /// [mnemonic] string of 12 words
  factory SolanaWallet.fromMnemonic(String mnemonic) {
    // Create the seed
    Uint8List seedBytes = bip39.mnemonicToSeed(mnemonic);
    // Create a private key from the seed
    KeyData keyData = ED25519_HD_KEY.derivePath("m/0'/501'", seedBytes);
    print(keyData.key.length);
    return SolanaWallet._fromKeyPair(keyData);
  }

  /// The address or public key of this wallet
  final String address;
  final KeyData _keyData;

  /// Returns a Future that resolves to the result of signing
  /// [data] with the private key held internally by a given
  /// instance
  Future<Signature> sign(List<int> data) {
    return ed25519.sign(
      data,
      ed25519.newKeyPairFromSeedSync(
        PrivateKey(_keyData.key.sublist(0, 32)),
      ),
    );
  }
}
